@@ -22,9 +22,10 @@ import wsgiref.util
 import requests
 import six
 
-from rollbar.lib import events, filters, dict_merge, parse_qs, text, transport, urljoin, iteritems
+from rollbar.lib import events, filters, dict_merge, parse_qs, text, transport, urljoin, iteritems, defaultJSONEncode
 
-__version__ = '0.14.2'
+
+__version__ = '0.15.0'
 __log_name__ = 'rollbar'
 log = logging.getLogger(__log_name__)
 
@@ -238,6 +239,7 @@ SETTINGS = {
         'accessToken',
         'auth',
         'authentication',
+        'authorization',
     ],
     'url_fields': ['url', 'link', 'href'],
     'notifier': {
@@ -259,6 +261,13 @@ SETTINGS = {
     'capture_username': False,
     'capture_ip': True,
     'log_all_rate_limited_items': True,
+    'http_proxy': None,
+    'http_proxy_user': None,
+    'http_proxy_password': None,
+    'include_request_body': False,
+    'request_pool_connections': None,
+    'request_pool_maxsize': None,
+    'request_max_retries': None,
 }
 
 _CURRENT_LAMBDA_CONTEXT = None
@@ -313,6 +322,7 @@ def init(access_token, environment='production', scrub_fields=None, url_fields=N
 
     SETTINGS['access_token'] = access_token
     SETTINGS['environment'] = environment
+    _configure_transport(**SETTINGS)
 
     if SETTINGS.get('allow_logging_basic_config'):
         logging.basicConfig()
@@ -364,6 +374,20 @@ def init(access_token, environment='production', scrub_fields=None, url_fields=N
     _initialized = True
 
 
+def _configure_transport(**kw):
+    configuration = _requests_configuration(**kw)
+    transport.configure_pool(**configuration)
+
+
+def _requests_configuration(**kw):
+    keys = {
+        'request_pool_connections': 'pool_connections',
+        'request_pool_maxsize': 'pool_maxsize',
+        'request_max_retries': 'max_retries',
+    }
+    return {keys[k]: kw.get(k, None) for k in keys}
+
+
 def lambda_function(f):
     """
     Decorator for making error handling on AWS Lambda easier
@@ -682,7 +706,7 @@ def _report_exc_info(exc_info, request, extra_data, payload_data, level=None):
         data = dict_merge(data, payload_data)
 
     payload = _build_payload(data)
-    send_payload(payload, data.get('access_token'))
+    send_payload(payload, payload.get('access_token'))
 
     return data['uuid']
 
@@ -690,11 +714,16 @@ def _report_exc_info(exc_info, request, extra_data, payload_data, level=None):
 def _walk_trace_chain(cls, exc, trace):
     trace_chain = [_trace_data(cls, exc, trace)]
 
+    seen_exceptions = {exc}
+
     while True:
         exc = getattr(exc, '__cause__', None) or getattr(exc, '__context__', None)
         if not exc:
             break
         trace_chain.append(_trace_data(type(exc), exc, getattr(exc, '__traceback__', None)))
+        if exc in seen_exceptions:
+            break
+        seen_exceptions.add(exc)
 
     return trace_chain
 
@@ -757,7 +786,7 @@ def _report_message(message, level, request, extra_data, payload_data):
         data = dict_merge(data, payload_data)
 
     payload = _build_payload(data)
-    send_payload(payload, data.get('access_token'))
+    send_payload(payload, payload.get('access_token'))
 
     return data['uuid']
 
@@ -820,11 +849,7 @@ def _build_person_data(request):
     """
     if hasattr(request, 'rollbar_person'):
         rollbar_person_prop = request.rollbar_person
-        try:
-            person = rollbar_person_prop()
-        except TypeError:
-            person = rollbar_person_prop
-
+        person = rollbar_person_prop() if callable(rollbar_person_prop) else rollbar_person_prop
         if person and isinstance(person, dict):
             return person
         else:
@@ -832,11 +857,7 @@ def _build_person_data(request):
 
     if hasattr(request, 'user'):
         user_prop = request.user
-        try:
-            user = user_prop()
-        except TypeError:
-            user = user_prop
-
+        user = user_prop() if callable(user_prop) else user_prop
         if not user:
             return None
         elif isinstance(user, dict):
@@ -860,11 +881,7 @@ def _build_person_data(request):
 
     if hasattr(request, 'user_id'):
         user_id_prop = request.user_id
-        try:
-            user_id = user_id_prop()
-        except TypeError:
-            user_id = user_id_prop
-
+        user_id = user_id_prop() if callable(user_id_prop) else user_id_prop
         if not user_id:
             return None
         return {'id': text(user_id)}
@@ -963,7 +980,10 @@ def _add_locals_data(trace_data, exc_info):
         if keywordspec:
             cur_frame['keywordspec'] = keywordspec
         if _locals:
-            cur_frame['locals'] = dict((k, _serialize_frame_data(v)) for k, v in iteritems(_locals))
+            try:
+                cur_frame['locals'] = dict((k, _serialize_frame_data(v)) for k, v in iteritems(_locals))
+            except Exception:
+                log.exception('Error while serializing frame data.')
 
         frame_num += 1
 
@@ -1121,14 +1141,25 @@ def _extract_wsgi_headers(items):
 
 
 def _build_django_request_data(request):
+    try:
+        url = request.get_raw_uri()
+    except AttributeError:
+        url = request.build_absolute_uri()
+
     request_data = {
-        'url': request.build_absolute_uri(),
+        'url': url,
         'method': request.method,
         'GET': dict(request.GET),
         'POST': dict(request.POST),
         'user_ip': _wsgi_extract_user_ip(request.META),
     }
 
+    if SETTINGS['include_request_body']:
+        try:
+            request_data['body'] = request.body
+        except:
+            pass
+
     request_data['headers'] = _extract_wsgi_headers(request.META.items())
 
     return request_data
@@ -1322,7 +1353,7 @@ def _build_payload(data):
 
 
 def _serialize_payload(payload):
-    return json.dumps(payload)
+    return json.dumps(payload, default=defaultJSONEncode)
 
 
 def _send_payload(payload_str, access_token):
@@ -1370,10 +1401,13 @@ def _post_api(path, payload_str, access_token=None):
 
     url = urljoin(SETTINGS['endpoint'], path)
     resp = transport.post(url,
-                         data=payload_str,
-                         headers=headers,
-                         timeout=SETTINGS.get('timeout', DEFAULT_TIMEOUT),
-                         verify=SETTINGS.get('verify_https', True))
+                          data=payload_str,
+                          headers=headers,
+                          timeout=SETTINGS.get('timeout', DEFAULT_TIMEOUT),
+                          verify=SETTINGS.get('verify_https', True),
+                          proxy=SETTINGS.get('http_proxy'),
+                          proxy_user=SETTINGS.get('http_proxy_user'),
+                          proxy_password=SETTINGS.get('http_proxy_password'))
 
     return _parse_response(path, SETTINGS['access_token'], payload_str, resp)
 
@@ -1382,7 +1416,12 @@ def _get_api(path, access_token=None, endpoint=None, **params):
     access_token = access_token or SETTINGS['access_token']
     url = urljoin(endpoint or SETTINGS['endpoint'], path)
     params['access_token'] = access_token
-    resp = transport.get(url, params=params, verify=SETTINGS.get('verify_https', True))
+    resp = transport.get(url,
+                         params=params,
+                         verify=SETTINGS.get('verify_https', True),
+                         proxy=SETTINGS.get('http_proxy'),
+                         proxy_user=SETTINGS.get('http_proxy_user'),
+                         proxy_password=SETTINGS.get('http_proxy_password'))
     return _parse_response(path, access_token, params, resp, endpoint=endpoint)
 
 
@@ -1440,12 +1479,16 @@ def _post_api_twisted(path, payload_str, access_token=None):
         r.headers.update(resp.headers.getAllRawHeaders())
         return treq.content(resp).addCallback(post_data_cb, r)
 
-    headers = {'Content-Type': ['application/json']}
+    headers = {'Content-Type': ['application/json; charset=utf-8']}
     if access_token is not None:
         headers['X-Rollbar-Access-Token'] = [access_token]
 
     url = urljoin(SETTINGS['endpoint'], path)
-    d = treq.post(url, payload_str, headers=headers,
+    try:
+        encoded_payload = payload_str.encode('utf8')
+    except (UnicodeDecodeError, UnicodeEncodeError):
+        encoded_payload = payload_str
+    d = treq.post(url, encoded_payload, headers=headers,
                   timeout=SETTINGS.get('timeout', DEFAULT_TIMEOUT))
     d.addCallback(post_cb)
 
