@@ -1,12 +1,17 @@
 import base64
 import copy
 import json
-import mock
 import socket
+import threading
 import uuid
 
 import sys
 
+try:
+    from unittest import mock
+except ImportError:
+    import mock
+
 try:
     from StringIO import StringIO
 except ImportError:
@@ -280,6 +285,67 @@ class RollbarTest(BaseTest):
         self.assertEqual(payload['data']['body']['trace_chain'][1]['exception']['class'], 'CauseException')
         self.assertEqual(payload['data']['body']['trace_chain'][1]['frames'][-1]['locals']['bar_local'], 'bar')
 
+    @mock.patch('rollbar.send_payload')
+    def test_report_exception_with_same_exception_as_cause(self, send_payload):
+        cause_exc = CauseException('bar')
+
+        def _raise_cause():
+            bar_local = 'bar'
+            raise cause_exc
+
+        def _raise_ex():
+            try:
+                _raise_cause()
+            except CauseException as cause:
+                # python2 won't automatically assign this traceback...
+                exc_info = sys.exc_info()
+                setattr(cause, '__traceback__', exc_info[2])
+
+                try:
+                    foo_local = 'foo'
+                    # in python3 this would normally be expressed as
+                    # raise cause from cause
+                    setattr(cause, '__cause__', cause)  # PEP-3134
+                    raise cause
+                except:
+                    rollbar.report_exc_info()
+
+        ex_raiser = threading.Thread(target=_raise_ex)
+        ex_raiser.daemon = True
+        ex_raiser.start()
+        # 0.5 seconds ought be enough for any modern computer to get into the
+        # cyclical parts of the code, but not so long as to collect a lot of
+        # objects in memory
+        ex_raiser.join(timeout=0.5)
+
+        if ex_raiser.is_alive():
+            # This breaks the circular reference, allowing thread to exit and
+            # to be joined
+            cause_exc.__cause__ = None
+            ex_raiser.join()
+            self.fail('Cyclic reference in rollbar._walk_trace_chain()')
+
+        self.assertEqual(send_payload.called, True)
+
+        payload = send_payload.call_args[0][0]
+
+        self.assertEqual(payload['access_token'], _test_access_token)
+        self.assertIn('body', payload['data'])
+        self.assertNotIn('trace', payload['data']['body'])
+        self.assertIn('trace_chain', payload['data']['body'])
+        self.assertEqual(2, len(payload['data']['body']['trace_chain']))
+
+        self.assertIn('exception', payload['data']['body']['trace_chain'][0])
+        self.assertEqual(payload['data']['body']['trace_chain'][0]['exception']['message'], 'bar')
+        self.assertEqual(payload['data']['body']['trace_chain'][0]['exception']['class'], 'CauseException')
+        frames = payload['data']['body']['trace_chain'][0]['frames']
+        self.assertEqual(payload['data']['body']['trace_chain'][0]['frames'][0]['locals']['foo_local'], 'foo')
+
+        self.assertIn('exception', payload['data']['body']['trace_chain'][1])
+        self.assertEqual(payload['data']['body']['trace_chain'][1]['exception']['message'], 'bar')
+        self.assertEqual(payload['data']['body']['trace_chain'][1]['exception']['class'], 'CauseException')
+        self.assertEqual(payload['data']['body']['trace_chain'][1]['frames'][-1]['locals']['bar_local'], 'bar')
+
     @mock.patch('rollbar.send_payload')
     def test_report_exception_with_context(self, send_payload):
         def _raise_context():
@@ -516,6 +582,22 @@ class RollbarTest(BaseTest):
         self.assertEqual('arg1', payload['data']['body']['trace']['frames'][-1]['argspec'][1])
         self.assertEqual(33, payload['data']['body']['trace']['frames'][-1]['locals']['arg1'])
 
+    @mock.patch('rollbar.send_payload')
+    def test_failed_locals_serialization(self, send_payload):
+
+        class tmp(object):
+            @property
+            def __class__(self):
+                foo()
+
+        try:
+            t = tmp()
+            raise Exception('trigger_serialize')
+        except:
+            rollbar.report_exc_info()
+
+        self.assertEqual(send_payload.called, True)
+
     @mock.patch('rollbar.send_payload')
     def test_args_lambda_no_args(self, send_payload):
 
@@ -604,7 +686,7 @@ class RollbarTest(BaseTest):
         varargs = payload['data']['body']['trace']['frames'][-1]['varargspec']
 
         self.assertEqual(1, len(payload['data']['body']['trace']['frames'][-1]['locals'][varargs]))
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][0], '\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][0], r'\*+')
 
     @mock.patch('rollbar.send_payload')
     def test_args_lambda_with_star_args_and_args(self, send_payload):
@@ -631,8 +713,8 @@ class RollbarTest(BaseTest):
         self.assertEqual('arg1-value', payload['data']['body']['trace']['frames'][-1]['locals']['arg1'])
 
         self.assertEqual(2, len(payload['data']['body']['trace']['frames'][-1]['locals'][varargs]))
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][0], '\*+')
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][1], '\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][0], r'\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][varargs][1], r'\*+')
 
     @mock.patch('rollbar.send_payload')
     def test_args_lambda_with_kwargs(self, send_payload):
@@ -795,7 +877,7 @@ class RollbarTest(BaseTest):
 
         self.assertEqual(2, len(payload['data']['body']['trace']['frames'][-1]['argspec']))
         self.assertEqual('password', payload['data']['body']['trace']['frames'][-1]['argspec'][0])
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['password'], '\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['password'], r'\*+')
         self.assertEqual('clear', payload['data']['body']['trace']['frames'][-1]['argspec'][1])
         self.assertEqual('text', payload['data']['body']['trace']['frames'][-1]['locals']['clear'])
 
@@ -849,7 +931,7 @@ class RollbarTest(BaseTest):
 
         self.assertEqual(2, len(payload['data']['body']['trace']['frames'][-1]['locals'][keywords]))
         self.assertIn('password', payload['data']['body']['trace']['frames'][-1]['locals'][keywords])
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][keywords]['password'], '\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals'][keywords]['password'], r'\*+')
         self.assertIn('clear', payload['data']['body']['trace']['frames'][-1]['locals'][keywords])
         self.assertEqual('text', payload['data']['body']['trace']['frames'][-1]['locals'][keywords]['clear'])
 
@@ -880,8 +962,8 @@ class RollbarTest(BaseTest):
 
         payload = send_payload.call_args[0][0]
 
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['password'], '\*+')
-        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['Password'], '\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['password'], r'\*+')
+        self.assertRegex(payload['data']['body']['trace']['frames'][-1]['locals']['Password'], r'\*+')
         self.assertIn('_invalid', payload['data']['body']['trace']['frames'][-1]['locals'])
 
         binary_type_name = 'str' if python_major_version() < 3 else 'bytes'
@@ -915,7 +997,7 @@ class RollbarTest(BaseTest):
             raise Exception()
 
         try:
-            obj = {}
+            obj = {'x': 42.3}
             obj['child'] = {
                 'parent': obj
             }
@@ -940,6 +1022,17 @@ class RollbarTest(BaseTest):
              payload['data']['body']['trace']['frames'][-1]['locals']['obj'].startswith('<CircularReference'))
         )
 
+        self.assertTrue(
+            (isinstance(payload['data']['body']['trace']['frames'][-1]['locals']['obj'], dict) and
+             'x' in payload['data']['body']['trace']['frames'][-1]['locals']['obj'] and
+             payload['data']['body']['trace']['frames'][-1]['locals']['obj']['x'] == 42.3)
+
+             or
+
+            (isinstance(payload['data']['body']['trace']['frames'][-1]['locals']['obj'], string_types) and
+             payload['data']['body']['trace']['frames'][-1]['locals']['obj'].startswith('<CircularReference'))
+        )
+
     @mock.patch('rollbar.send_payload')
     def test_scrub_local_ref(self, send_payload):
         """
